<#
.SYNOPSIS
File:         GPOinstaller.ps1
System:       vistrax
Component:    
Version:      0.1.0

Legal:        Copyright (c) CONZE Informatik GmbH
              All other rights reserved.
.Description
This script installs vistrax according to GPO-Guidelines. 
STILL IN DEVELOPMENT
.Example
.\GPOinstaller.ps1
.EXAMPLE
.\GPOinstaller.ps1 -installMode:"/qn" 
.EXAMPLE
GPOinstaller.ps1 -installerPath:"\\[DomainServer]\Netlogon" -silentConfig -installMode:"/qn" -vxDbServer:"192.168.2.111\VISTRAX" -vxLdapAdsDomain:"core.domain.xx" -vxLdapAdsPath:"ldap://core.domain.xx"
#>

# ----------------------------------------------------------------------------
# Parameter
# ----------------------------------------------------------------------------

[CmdletBinding()]
param
(
    #set this bool, if the database connection should be configured silently
    [bool] $silentConfig=$true,  
    #set this value to "/qb" or "/passive" for passive or "/qn" "/quiet" for quiet/silent
    [string] $installMode = "/qb", 
    #set installer language code, currently supported 1033 (english) and 1031 (german)
    [int] $installerLanguage = 1033,
    #version, this script should install
    [string] $installerVersion = '##version##',
    #["server"|"windows"] default="server"    
    [string] $vxDbAuthMode="server",      
    #[hostname|ip] default ="127.0.0.1\VISTRAX"
    [string] $vxDbServer="127.0.0.1\VISTRAX",   
    #default = 1433
    [int] $vxDbPort = 1433,                 
    #Database name, default = "VISTRAX"
    [string] $vxDbname = "VISTRAX",       
    [string] $vxDbuser = "SA",         
    [bool] $vxUseEncryptedPassword=$true,
    [string] $vxDbPassword = "VxAdmin4DB!", 
    [string] $vxDbEncryptedPassword = "79291ceaf708f91b2bf3096ad8c44cbfb9bf3a16fe0eaa21",
    #["msoledbsql"|"sqloledb"]
    [string] $vxDbDriver = "sqloledb",
    #["disabled"|"auto"|"forced"|"forcednocert"]
    [string] $vxDbEncryption = "disabled",
    #default = 4096
    [int] $vxDbPacketSize = 4096,

    [string] $vxLdapAdsPath, 
    [string] $vxLdapAdsDomain,
    #use, if a License at Path=vxLicensePath should be loaded
    [bool] $vxLoadLicense=$false,
    #path to a .vxl file [default = Documents\vistraxLicense.vxl]
    [string] $vxLicensePath = "\\[DomainServer]\Netlogon\vistraxLicense.vxl",
    #if the script is not in your current path, enter the scripts path
    [string] $InstallerPath = "."

)

# ----------------------------------------------------------------------------
# Variables
# ----------------------------------------------------------------------------

#Used to save states of installations
#vistrax tracks installation-status of vistrax
#path tracks installation-directory of vistrax
#Needed is set, if vistrax needs to be installed
[hashtable] $Installation = @{"vistrax"=$false; "path"=$null; "Needed"=$true}
$processName = "vistrax_setup_$installerVersion"
$installerName = "$processName.exe"
# ----------------------------------------------------------------------------
# Check Vistrax Installation
# ----------------------------------------------------------------------------

function Get-Version([String] $message)
{    
    Write-Output $message`n | timestamp | Tee-Object -a $logDirectory
    $vistraxVersion = (Get-ItemProperty $regPath -ErrorAction SilentlyContinue).Version 
    #$vistraxInstalledSettings = (Get-ItemProperty  "Registry::HKEY_LOCAL_MACHINE\SOFTWARE\$64bitRegistryPath`CONZE Informatik\vistrax\settings" -ErrorAction SilentlyContinue).Language #not null if vistrax is installed    

    # Check if version is greater than installed version    

    $InstalledVersion = [Version]$vistraxVersion | Convert-VersionToComparableText
    $ScriptVersion = [Version]$installerVersion | Convert-VersionToComparableText
    if ($InstalledVersion) { $Installation.Needed = $ScriptVersion -gt $InstalledVersion } 

    #print and set installation and path

    $Version = "Version:`t"
    if (!$vistraxVersion)
    {
        Write-Output "No Vistrax installed." | timestamp | Tee-Object -a $logDirectory
    } 
    else 
    {   
        #Generate Installation Status Output
        # ----------------------------------------------------------------------------
        $Version += $vistraxVersion 
        $Version += ({ => no Update needed.}, { => Update needed.})[$Installation.Needed] 
        #save global boolean, indicating if vistrax is installed
        $Installation.Vistrax = ({$true},{$false})[!$vistraxVersion]        
        Write-Output $Version | timestamp | Tee-Object -a $logDirectory
        #save vistrax directory to global var
        $Installation.Path =(Get-ItemProperty $regPath).Path       
        $location = "Location:`t" 
        $location += $Installation.Path +"`n"
        Write-Output $location | timestamp | Tee-Object -a $logDirectory        
        # ----------------------------------------------------------------------------
    }    
}

# ----------------------------------------------------------------------------
# Functions
# ----------------------------------------------------------------------------

#Converts parameters for use in process call and writes them to a hashtable
#Adding double-quotes before and after the value to represent as a string in process-call later
function paramGetValues 
{
    [hashtable] $connection = @{}
    
    $connection.vxDatabase              = ({"silent"},{"config"})[!$silentConfig]
    $connection.vxDbauthmode            = '"' + $vxDbauthmode + '"'
    $connection.vxDbserver              = '"' + $vxDbserver + '"'
    $connection.vxDbport                =       $vxDbport                
    $connection.vxDbname                = '"' + $vxDbname + '"'         
    $connection.vxDbuser                = '"' + $vxDbuser + '"'    
    $connection.vxUseEncryptedPassword  =       $vxUseEncryptedPassword
    $connection.vxDbpassword            = '"' + $vxDbpassword + '"'
    $connection.vxDbencryptedpassword   = '"' + $vxDbencryptedpassword + '"'
    $connection.vxDbDriver              = '"' + $vxDbDriver + '"'
    $connection.vxDbencryption          = '"' + $vxDbencryption + '"'
    $connection.vxDbpacketsize          = '"' + $vxDbpacketsize + '"'

    $connection.vxLdapAdsPath           = '"' + $vxLdapAdsPath + '"'
    $connection.vxLdapAdsDomain         = '"' + $vxLdapAdsDomain + '"'

    $connection.vxLicensePath           = '"' + $vxLicensePath + '"'

    return $connection
}

#calls vistrax to set database connection details according to connection hashtable
function Configure-Vistrax
{
    Write-Output "Configuring vistrax, please wait." | timestamp | Tee-Object -a $logDirectory
    #Combine Parameters to a single expression
    # ----------------------------------------------------------------------------
    $params = '-database:'+ $connection.vxDatabase `
            +' -dbauthmode:'+ $connection.vxDbauthmode `
            +' -dbserver:'+ $connection.vxDbserver `
            +' -dbport:'+ $connection.vxDbport `
            +' -dbname:'+ $connection.vxDbname `
            +' -dbuser:'+ $connection.vxDbuser `
            +' -dbdriver:'+ $connection.vxDbDriver `
            +' -dbencryption:'+ $connection.vxDbencryption `
            +' -dbpacketsize:'+ $connection.vxDbpacketsize 
        
    if ($vxLdapAdsDomain)
    {
        $params += ' -ldapadsdomain:'+ $connection.vxLdapAdsDomain
    }
    if ($vxLdapAdsPath)
    {
        $params += ' -ldapadspath:'+ $connection.vxLdapAdsPath
        
    }
    $paramspw = ' -dbpassword:'+ $connection.vxDbpassword
    $paramsencryptedpw = '-dbcryptedpassword:'+ $connection.vxDbencryptedpassword
    # ----------------------------------------------------------------------------

    #Call vistrax to set connection settings
    if ($installation.vistrax) 
    {
        
        if (!$connection.vxUseEncryptedPassword)
        {
            & (Join-Path -Path $Installation.Path -ChildPath "vistrax\vistrax.exe") $params $paramspw
        }
        else
        {
            & (Join-Path -Path $Installation.Path -ChildPath "vistrax\vistrax.exe") $params $paramsencryptedpw
        }
        $output = "Configured vistrax " 
        $output += ({with}, {without})[!$connection.vxUseEncryptedPassword] 
        $output += " encrypted password."
        Write-Output $output  | timestamp | Tee-Object -a $logDirectory
    }
}
#load .vxl license file
function Load-License 
{
    if ($Installation.vistrax) 
    {
        Write-Output ("Trying to load license file at "+$vxLicensePath) | timestamp | Tee-Object -a $logDirectory
        if (!(Test-Path($vxLicensePath)))
        {
            Write-Output "ERROR: No license file found." | timestamp | Tee-Object -a $logDirectory
        }
        else
        {
            Write-Output "Loading license file..." | timestamp | Tee-Object -a $logDirectory
            & $Installation.Path\vistrax\vistrax.exe -license:$connection.vxLicensePath
            Write-Output "License file loaded." | timestamp | Tee-Object -a $logDirectory
        }
    }
    else 
    {
        Write-Output "`nERROR: Vistrax is not installed, License cant be loaded" | timestamp | Tee-Object -a $logDirectory
    }
}

#calls vistrax installation process
function Install-Vistrax
{

    #build installtion parameters accoring to script parameters
    $installMode = "'$installMode'"
    $installerLanguage = "/exelang $installerLanguage"

    #Add custom script path if not called from within its directory
    if (!$InstallerPath)
    {
        $InstallerPath = ".\"
    }
    $exe = Get-Item (Join-Path -Path $InstallerPath -ChildPath $installerName) -ErrorAction Stop | Tee-Object -Append $logDirectory
    if (!$exe)
    {
        Write-Output "Installer not found." | timestamp | Tee-Object -a $logDirectory
        exit
    }
    #copy installer to temp and build exe-name
    Write-Output "Copying Installer from $exe to $env:TEMP." | timestamp | Tee-Object -a $logDirectory
    Copy-Item -Path $exe -Destination $env:TEMP -ErrorAction Stop
    Write-Output "Done." | timestamp | Tee-Object -a $logDirectory
    $exe = Join-Path -Path $env:TEMP -ChildPath $installerName

    #added to generate return
    $errorHandlingSuffix = ';$?'

    $output = "Installing Vistrax "+ $installerVersion +" Please Wait... "
    Write-Output $output | timestamp | Tee-Object -a $logDirectory

    #call installer
    $Success = Invoke-Expression -Command "&$exe $installerLanguage $installMode $errorHandlingSuffix" | timestamp | Tee-Object -a $logDirectory

    #Print Installer Activity
    $setup = Get-Process $processName -ErrorAction SilentlyContinue
    $chainer = Get-Process "aipackagechainer" -ErrorAction SilentlyContinue

    while ($setup -or $chainer) {
        Write-Host -NoNewLine "`rInstalling /" | timestamp
        Start-Sleep -Milliseconds 100
        Write-Host -NoNewLine "`rInstalling -" | timestamp
        Start-Sleep -Milliseconds 100
        Write-Host -NoNewLine "`rInstalling \" | timestamp
        Start-Sleep -Milliseconds 100
        Write-Host -NoNewLine "`rInstalling |" | timestamp
        Start-Sleep -Milliseconds 100
        $setup = Get-Process $processName -ErrorAction SilentlyContinue
        $chainer = Get-Process "aipackagechainer" -ErrorAction SilentlyContinue
    }

    #print result
    if (!$Success) 
    { 
        Write-Output "`r                    "
        Write-Output "ERROR: Installation failed."
    } 
        else 
    { 
        Write-Output "`r                     "
        Write-Output "Installation finished." | timestamp | Tee-Object -a $logDirectory
    }
}
#creates log-file and directory
function Configure-Logging
{
    #Set logfile directory
    $logDirectory = Join-Path -Path $env:APPDATA -ChildPath '\CONZE Informatik\InstallerScript'
    #check if directory exists, if not, create one
    if(!(Test-Path $logDirectory)){New-Item -ItemType Directory -Force -Path $logDirectory | Out-Null } 
    #append logfile to path
    $logDirectory = Join-Path -Path $logDirectory -ChildPath 'log.txt' 
    #check if logfile already exists, if not, create one
    if(!(Test-Path $logDirectory)){New-Item -ItemType File -Force -Path $logDirectory | Out-Null} 
    #wait until file is created
    while (!(Test-Path $logDirectory)) { Start-Sleep 10 
    Write-Output "Waiting"}
    return $logDirectory    
}

function Check-PSVersion{
    $PSVersion = $PSVersionTable.PSVersion | Convert-VersionToComparableText
    if (!($PSVersion -ge "0000000003.0000000000.0000000000.0000000000"))
    {
        $Output = "Powershell Version only "+$PSVersionTable.PSVersion+". Version 3.0 is needed." 
        Write-Output $Output| timestamp 
        exit
    }
}

# ----------------------------------------------------------------------------
# Filter
# ----------------------------------------------------------------------------

#Converts Input to format (DD.MM.YYYY hh:mm:ss: $INPUT)
filter timestamp {"$(Get-Date -Format G): $_"}

#Converts a version to a comparable text
filter Convert-VersionToComparableText { '{0:0000000000}{1:0000000000}{2:0000000000}{3:0000000000}' -f $_.Major, $_.Minor, $_.Build, $_.Revision }

# ----------------------------------------------------------------------------
# Core
# ----------------------------------------------------------------------------
Check-PSVersion
$logDirectory = Configure-Logging

#empty line in logfile indicating new entry
" " | Tee-Object $logDirectory

#Edit Registry Path according to Operating System
$64bitRegistryPath = ({WOW6432Node\},{})[![System.Environment]::Is64BitOperatingSystem]
$regPath = "Registry::HKEY_LOCAL_MACHINE\SOFTWARE\$64bitRegistryPath`CONZE Informatik\vistrax"

#header
Write-Output "Vistrax Installation Script`n" | timestamp | Tee-Object -Append $logDirectory
#Check current installation of vistrax
Get-Version('Checking current Version...') 

#edit parameteres 
$connection = paramGetValues

if ($Installation.Needed)
{
    Install-Vistrax 
    Get-Version('Checking Installation...')
    Configure-Vistrax
}

if ($vxLoadLicense) { Load-License }

#Remove Installer from Temp
Write-Output "Removing Installer from '$env:TEMP'..." | timestamp | Tee-Object -a $logDirectory
Remove-Item (Join-Path -Path $env:TEMP -ChildPath $installerName) -ErrorAction Continue
Write-Output "Done." | timestamp | Tee-Object -a $logDirectory

Write-Output "Finished." | timestamp | Tee-Object -a $logDirectory
